@@ -2,26 +2,26 @@
 
 from __future__ import unicode_literals
 
-import operator
 import os
 
-from pyte import HistoryScreen, Stream, control as ctrl, modes as mo
-from pyte.compat import map, str
+import pyte
+from pyte import control as ctrl, modes as mo
+from pyte.compat import str
 
 
-def chars(lines):
-    return ["".join(map(operator.attrgetter("data"), line))
-            for line in lines]
+def chars(history_lines, columns):
+    return ["".join(history_lines[y][x].data for x in range(columns))
+            for y in range(len(history_lines))]
 
 
 def test_index():
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
 
     # Filling the screen with line numbers, so it's easier to
     # track history contents.
-    for idx in range(len(screen.buffer)):
+    for idx in range(screen.lines):
         screen.draw(str(idx))
-        if idx != len(screen.buffer) - 1:
+        if idx != screen.lines - 1:
             screen.linefeed()
 
     assert not screen.history.top
@@ -40,14 +40,14 @@ def test_index():
     assert screen.history.top[-1] == line
 
     # c) rotation.
-    for _ in range(len(screen.buffer) * screen.lines):
+    for _ in range(screen.history.size * 2):
         screen.index()
 
-    assert len(screen.history.top) == 25  # pages // 2 * lines
+    assert len(screen.history.top) == 50
 
 
 def test_reverse_index():
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
 
     # Filling the screen with line numbers, so it's easier to
     # track history contents.
@@ -81,23 +81,20 @@ def test_reverse_index():
 
 
 def test_prev_page():
-    screen = HistoryScreen(4, 4, history=40)
+    screen = pyte.HistoryScreen(4, 4, history=40)
     screen.set_mode(mo.LNM)
 
     assert screen.history.position == 40
 
     # Once again filling the screen with line numbers, but this time,
     # we need them to span on multiple lines.
-    for idx in range(len(screen.buffer) * 10):
-        for ch in str(idx):
-            screen.draw(ch)
-
+    for idx in range(screen.lines * 10):
+        screen.draw(str(idx))
         screen.linefeed()
 
     assert screen.history.top
     assert not screen.history.bottom
     assert screen.history.position == 40
-    assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "37  ",
         "38  ",
@@ -105,7 +102,7 @@ def test_prev_page():
         "    "
     ]
 
-    assert chars(screen.history.top)[-4:] == [
+    assert chars(screen.history.top, screen.columns)[-4:] == [
         "33  ",
         "34  ",
         "35  ",
@@ -114,7 +111,7 @@ def test_prev_page():
 
     # a) first page up.
     screen.prev_page()
-    assert screen.history.position == 36
+    assert screen.history.position == 38
     assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "35  ",
@@ -123,7 +120,7 @@ def test_prev_page():
         "38  "
     ]
 
-    assert chars(screen.history.top)[-4:] == [
+    assert chars(screen.history.top, screen.columns)[-4:] == [
         "31  ",
         "32  ",
         "33  ",
@@ -131,14 +128,14 @@ def test_prev_page():
     ]
 
     assert len(screen.history.bottom) == 2
-    assert chars(screen.history.bottom) == [
+    assert chars(screen.history.bottom, screen.columns) == [
         "39  ",
         "    ",
     ]
 
     # b) second page up.
     screen.prev_page()
-    assert screen.history.position == 32
+    assert screen.history.position == 36
     assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "33  ",
@@ -148,7 +145,7 @@ def test_prev_page():
     ]
 
     assert len(screen.history.bottom) == 4
-    assert chars(screen.history.bottom) == [
+    assert chars(screen.history.bottom, screen.columns) == [
         "37  ",
         "38  ",
         "39  ",
@@ -156,13 +153,11 @@ def test_prev_page():
     ]
 
     # c) same with odd number of lines.
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
     screen.set_mode(mo.LNM)
 
-    for idx in range(len(screen.buffer) * 10):
-        for ch in str(idx):
-            screen.draw(ch)
-
+    for idx in range(screen.lines * 10):
+        screen.draw(str(idx))
         screen.linefeed()
 
     assert screen.history.top
@@ -177,8 +172,7 @@ def test_prev_page():
     ]
 
     screen.prev_page()
-    assert screen.history.position == 45
-    assert len(screen.buffer) == screen.lines
+    assert screen.history.position == 47
     assert screen.display == [
         "43   ",
         "44   ",
@@ -188,20 +182,52 @@ def test_prev_page():
     ]
 
     assert len(screen.history.bottom) == 3
-    assert chars(screen.history.bottom) == [
+    assert chars(screen.history.bottom, screen.columns) == [
         "48   ",
         "49   ",
         "     ",
     ]
 
-    # d) same with cursor in the middle of the screen.
-    screen = HistoryScreen(5, 5, history=50)
+    # d) with a ratio other than 0.5
+    screen = pyte.HistoryScreen(4, 4, history=40, ratio=0.75)
     screen.set_mode(mo.LNM)
 
-    for idx in range(len(screen.buffer) * 10):
-        for ch in str(idx):
-            screen.draw(ch)
+    for idx in range(screen.lines * 10):
+        screen.draw(str(idx))
+        screen.linefeed()
 
+    assert screen.history.top
+    assert not screen.history.bottom
+    assert screen.history.position == 40
+    assert screen.display == [
+        "37  ",
+        "38  ",
+        "39  ",
+        "    "
+    ]
+
+    screen.prev_page()
+    assert screen.history.position == 37
+    assert screen.display == [
+        "34  ",
+        "35  ",
+        "36  ",
+        "37  "
+    ]
+
+    assert len(screen.history.bottom) == 3
+    assert chars(screen.history.bottom, screen.columns) == [
+        "38  ",
+        "39  ",
+        "    "
+    ]
+
+    # e) same with cursor in the middle of the screen.
+    screen = pyte.HistoryScreen(5, 5, history=50)
+    screen.set_mode(mo.LNM)
+
+    for idx in range(screen.lines * 10):
+        screen.draw(str(idx))
         screen.linefeed()
 
     assert screen.history.top
@@ -223,18 +249,17 @@ def test_prev_page():
     assert screen.history.position == screen.lines
     assert len(screen.buffer) == screen.lines
     assert screen.display == [
-        "21   ",
-        "22   ",
-        "23   ",
-        "24   ",
-        "25   "
+        "1    ",
+        "2    ",
+        "3    ",
+        "4    ",
+        "5    "
     ]
 
     while screen.history.position < screen.history.size:
         screen.next_page()
 
     assert screen.history.position == screen.history.size
-    assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "46   ",
         "47   ",
@@ -244,12 +269,11 @@ def test_prev_page():
     ]
 
     # e) same with cursor near the middle of the screen.
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
     screen.set_mode(mo.LNM)
 
-    for idx in range(len(screen.buffer) * 10):
-        for ch in str(idx):
-            screen.draw(ch)
+    for idx in range(screen.lines * 10):
+        screen.draw(str(idx))
 
         screen.linefeed()
 
@@ -272,11 +296,11 @@ def test_prev_page():
     assert screen.history.position == screen.lines
     assert len(screen.buffer) == screen.lines
     assert screen.display == [
-        "21   ",
-        "22   ",
-        "23   ",
-        "24   ",
-        "25   "
+        "1    ",
+        "2    ",
+        "3    ",
+        "4    ",
+        "5    "
     ]
 
     while screen.history.position < screen.history.size:
@@ -294,21 +318,18 @@ def test_prev_page():
 
 
 def test_next_page():
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
     screen.set_mode(mo.LNM)
 
     # Once again filling the screen with line numbers, but this time,
     # we need them to span on multiple lines.
-    for idx in range(len(screen.buffer) * 5):
-        for ch in str(idx):
-            screen.draw(ch)
-
+    for idx in range(screen.lines * 5):
+        screen.draw(str(idx))
         screen.linefeed()
 
     assert screen.history.top
     assert not screen.history.bottom
     assert screen.history.position == 50
-    assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "21   ",
         "22   ",
@@ -335,15 +356,14 @@ def test_next_page():
     screen.prev_page()
     screen.prev_page()
     screen.next_page()
-    assert screen.history.position == 45
+    assert screen.history.position == 47
     assert screen.history.top
-    assert chars(screen.history.bottom) == [
+    assert chars(screen.history.bottom, screen.columns) == [
         "23   ",
         "24   ",
         "     "
     ]
 
-    assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "18   ",
         "19   ",
@@ -357,7 +377,7 @@ def test_next_page():
     screen.prev_page()
     screen.next_page()
     screen.next_page()
-    assert screen.history.position == 45
+    assert screen.history.position == 47
     assert len(screen.buffer) == screen.lines
     assert screen.display == [
         "18   ",
@@ -369,65 +389,46 @@ def test_next_page():
 
 
 def test_ensure_width(monkeypatch):
-    screen = HistoryScreen(5, 5, history=50)
+    screen = pyte.HistoryScreen(5, 5, history=50)
     screen.set_mode(mo.LNM)
+    escape = dict(pyte.Stream.escape)
+    escape.update({"N": "next_page", "P": "prev_page"})
+    monkeypatch.setattr(pyte.Stream, "escape", escape)
 
-    monkeypatch.setattr(Stream, "escape",
-                        dict(N="next_page", P="prev_page", **Stream.escape))
-    stream = Stream()
-    stream.attach(screen)
+    stream = pyte.Stream(screen)
 
-    for idx in range(len(screen.buffer) * 5):
-        for ch in str(idx) + os.linesep:
-            stream.feed(ch)
+    for idx in range(screen.lines * 5):
+        stream.feed("{0:04d}".format(idx) + os.linesep)
 
     assert screen.display == [
-        "21   ",
-        "22   ",
-        "23   ",
-        "24   ",
+        "0021 ",
+        "0022 ",
+        "0023 ",
+        "0024 ",
         "     "
     ]
 
-    # a) shrinking the screen, expecting the lines displayed to
-    #    be truncated.
-    screen.resize(5, 2)
+    # Shrinking the screen should truncate the displayed lines following lines.
+    screen.resize(5, 3)
     stream.feed(ctrl.ESC + "P")
 
-    assert all(len(l) != 2 for l in screen.history.top)
-    assert all(len(l) == 2 for l in screen.history.bottom)
+    # Inequality because we have an all-empty last line.
+    assert all(len(l) <= 3 for l in screen.history.bottom)
     assert screen.display == [
-        "18",
-        "19",
-        "20",
-        "21",
-        "22"
-    ]
-
-    # b) expading the screen, expecting the lines displayed to
-    #    be filled with whitespace characters.
-    screen.resize(5, 10)
-    stream.feed(ctrl.ESC + "N")
-
-    assert all(len(l) == 10 for l in list(screen.history.top)[-3:])
-    assert all(len(l) != 10 for l in screen.history.bottom)
-    assert screen.display == [
-        "21        ",
-        "22        ",
-        "23        ",
-        "24        ",
-        "          "
+        "001",  # 18
+        "001",  # 19
+        "002",  # 20
+        "002",  # 21
+        "002"   # 22
     ]
 
 
 def test_not_enough_lines():
-    screen = HistoryScreen(5, 5, history=6)
+    screen = pyte.HistoryScreen(5, 5, history=6)
     screen.set_mode(mo.LNM)
 
-    for idx in range(len(screen.buffer)):
-        for ch in str(idx):
-            screen.draw(ch)
-
+    for idx in range(screen.lines):
+        screen.draw(str(idx))
         screen.linefeed()
 
     assert screen.history.top
@@ -444,7 +445,7 @@ def test_not_enough_lines():
     screen.prev_page()
     assert not screen.history.top
     assert len(screen.history.bottom) is 1
-    assert chars(screen.history.bottom) == ["     "]
+    assert chars(screen.history.bottom, screen.columns) == ["     "]
     assert screen.display == [
         "0    ",
         "1    ",
@@ -465,17 +466,16 @@ def test_not_enough_lines():
     ]
 
 
-def test_draw():
-    screen = HistoryScreen(5, 5, history=50)
+def test_draw(monkeypatch):
+    screen = pyte.HistoryScreen(5, 5, history=50)
     screen.set_mode(mo.LNM)
-    stream = Stream()
-    stream.attach(screen)
-    stream.escape["N"] = "next_page"
-    stream.escape["P"] = "prev_page"
+    escape = dict(pyte.Stream.escape)
+    escape.update({"N": "next_page", "P": "prev_page"})
+    monkeypatch.setattr(pyte.Stream, "escape", escape)
 
-    for idx in range(len(screen.buffer) * 5):
-        for ch in str(idx) + os.linesep:
-            stream.feed(ch)
+    stream = pyte.Stream(screen)
+    for idx in range(screen.lines * 5):
+        stream.feed(str(idx) + os.linesep)
 
     assert screen.display == [
         "21   ",
@@ -501,16 +501,15 @@ def test_draw():
     ]
 
 
-def test_cursor_is_hidden():
-    screen = HistoryScreen(5, 5, history=50)
-    stream = Stream()
-    stream.attach(screen)
-    stream.escape["N"] = "next_page"
-    stream.escape["P"] = "prev_page"
+def test_cursor_is_hidden(monkeypatch):
+    screen = pyte.HistoryScreen(5, 5, history=50)
+    escape = dict(pyte.Stream.escape)
+    escape.update({"N": "next_page", "P": "prev_page"})
+    monkeypatch.setattr(pyte.Stream, "escape", escape)
 
-    for idx in range(len(screen.buffer) * 5):
-        for ch in str(idx) + os.linesep:
-            stream.feed(ch)
+    stream = pyte.Stream(screen)
+    for idx in range(screen.lines * 5):
+        stream.feed(str(idx) + os.linesep)
 
     assert not screen.cursor.hidden
 
@@ -522,3 +521,19 @@ def test_cursor_is_hidden():
     assert screen.cursor.hidden
     stream.feed(ctrl.ESC + "N")
     assert not screen.cursor.hidden
+
+
+def test_erase_in_display():
+    screen = pyte.HistoryScreen(5, 5, history=6)
+    screen.set_mode(mo.LNM)
+
+    for idx in range(screen.lines):
+        screen.draw(str(idx))
+        screen.linefeed()
+
+    screen.prev_page()
+
+    # See #80 on GitHub for details.
+    screen.erase_in_display(3)
+    assert not screen.history.top
+    assert not screen.history.bottom
